/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung;
import java.util.Collection;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;

/**
 * Schnittstellendefinition für Konvertierungsklassen zur Übersetzung von statischen Daten
 * vom Datenverteiler-Format in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */
public interface DatexKonvertierungStatisch { 
  /**
   * Optionen bei der Erzeugung.
   * 
   */
  public enum CreationOptions {
    /** Setzen aller Felder, die eine Versionsangabe beinhalten, auf feste Standard-Werte. */
    VERSION_FREE
  }

  /**
   * Führe Konvertierung von Datenverteiler-Daten nach DATEX-II durch.
   * 
   * @param objekteAtgAspListe Liste von Objektliste/Attributgruppe/Aspekt-Kombinationen.
   * @param options Entferne alle Versionshinweise in der Datex-Struktur
   * @return JAX-B Struktur (Kontext über getJAXBContext() abrufbar)
   */
  Object getDatexII(Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options);
  
  /**
   * Rückgabe des JAX-B-Kontexts für die in liefereDaten verwendete JAX-B-Struktur.
   * 
   * @return JAXBContext
   * @throws JAXBException Falls die Instantiierung des Kontext nicht möglich ist.
   */
  JAXBContext getJAXBContext() throws JAXBException;
  
  /**
   * Initialisiere die Instanz mit einer Liste von Objektliste/Attributgruppe/Aspekt-Kombination.
   * 
   * @param objects Liste von Objektliste/Attributgruppe/Aspekt-Kombination.
   * @throws ConfigurationException Fehler bei der Initialisierung (bspw. fehlende Parameter)
   */
  void initialisiere(Collection<ObjekteAtgAsp> objects) throws ConfigurationException;

}
