/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import eu.datex2.schema.dynamisch.baustellen.DistanceFromLinearElementReferent;
import eu.datex2.schema.dynamisch.baustellen.LinearElement;
import eu.datex2.schema.dynamisch.baustellen.LinearWithinLinearElement;
import eu.datex2.schema.dynamisch.baustellen.Referent;
import eu.datex2.schema.dynamisch.baustellen.ReferentTypeEnum;

/**
 * Wrapper f&uuml;r {@link LinearWithinLinearElement}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public class Datex2AsbLocationLinear extends LinearWithinLinearElement {

	/**
	 * Konstruktor.
	 * 
	 * @param data
	 *            {@link Array}
	 * @throws Datex2BaustelleException
	 *             bei Fehlern
	 */
	public Datex2AsbLocationLinear(final Array data) throws Datex2BaustelleException {
		super();
		final Data item = data.getItem(0);

		final DistanceFromLinearElementReferent fromPoint = new DistanceFromLinearElementReferent();
		final String vonAbschnitt = item.getItem("VonAbschnitt").asTextValue().getText();
		buildReferent(fromPoint, vonAbschnitt);
		fromPoint.setDistanceAlong(item.getItem("VonStation").asUnscaledValue().floatValue());

		final DistanceFromLinearElementReferent toPoint = new DistanceFromLinearElementReferent();
		final String bisAbschnitt = item.getItem("BisAbschnitt").asTextValue().getText();
		buildReferent(toPoint, bisAbschnitt);
		toPoint.setDistanceAlong(item.getItem("BisStation").asUnscaledValue().floatValue());

		// Inhalt LinearElement entfaellt lt. Absprache da keine passende
		// Modellierung im DatK.
		final LinearElement linearElement = new LinearElement();
		setLinearElement(linearElement);

		setFromPoint(fromPoint);
		setToPoint(toPoint);
	}

	private void buildReferent(final DistanceFromLinearElementReferent point, final String asbabschnitt)
			throws Datex2BaustelleException {
		// <kurzinfo>Abschnitt des Anfangspunktes des referenzierten Bereichs.
		// Die Angabe für Abschnitte muss entweder als'NP-NP' oder als 'NPNP'
		// (16-stellig) (z. B.
		// '5506024O-5506030A' oder '5506024O5506030A' erfolgen.
		// </kurzinfo>

		final String pattern = "[0-9]{7}[A-Z]-?[0-9]{7}[A-Z]";

		if (!asbabschnitt.matches(pattern)) {
			throw new Datex2BaustelleException("Die Angabe des ASB-Abschnittes '" + asbabschnitt + "' ist ungueltig!");
		}

		final String fromNP;
		final String nachNP;
		if (asbabschnitt.contains("-")) {
			final String[] split = asbabschnitt.split("-");
			fromNP = split[0];
			nachNP = split[1];
		} else {
			fromNP = asbabschnitt.substring(0, 8);
			nachNP = asbabschnitt.substring(8);
		}

		point.setFromReferent(buildReferent(fromNP));
		point.setTowardsReferent(buildReferent(nachNP));
	}

	private Referent buildReferent(final String nullpunkt) {
		final Referent referent = new Referent();
		referent.setReferentIdentifier(nullpunkt);
		referent.setReferentType(ReferentTypeEnum.REFERENCE_MARKER);
		return referent;
	}
}
