/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ActionStatusEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ArtDerMassnahmeEnum;
import eu.datex2.schema.dynamisch.baustellen.OperatorAction;
import eu.datex2.schema.dynamisch.baustellen.OperatorActionExtended;
import eu.datex2.schema.dynamisch.baustellen.OperatorActionExtensionType;
import eu.datex2.schema.dynamisch.baustellen.SituationRecord;

/**
 * Wrapper f&uuml;r {@link OperatorAction}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public abstract class Datex2OperatorAction extends Datex2SituationRecord {

	/**
	 * Konstruktor.
	 * 
	 * @param situationRecord
	 *            {@link SituationRecord}
	 * @param attprefix
	 *            Attribut-Prefix
	 * @param resultData
	 *            {@link ResultData}
	 * @param data
	 *            {@link Data}
	 */
	public Datex2OperatorAction(final SituationRecord situationRecord, final String attprefix,
			final ResultData resultData, final Data data) {
		super(situationRecord, attprefix, resultData, data);
	}

	@Override
	public SituationRecord getSituationRecord() throws Datex2BaustelleException {
		final SituationRecord situationRecord = super.getSituationRecord();
		final OperatorAction operatorAction = (OperatorAction) situationRecord;

		// actionPlanIdentifier
		Data item = getData().getItem(getDavAttributPrefix() + "ArtDerMassnahme");
		operatorAction.setActionPlanIdentifier(
				Datex2ArtDerMassnahmeEnum.getInstance(item.asUnscaledValue().intValue()).getActionPlanIdentifier());

		// operatorActionStatus (optional)
		item = getData().getItem(getDavAttributPrefix() + "Status");
		final Datex2ActionStatusEnum status = Datex2ActionStatusEnum.getInstance(item.asUnscaledValue().intValue());
		if (status.getDatexWert() != null) {
			operatorAction.setOperatorActionStatus(status.getDatexWert());
		}

		// operatorActionExtended (nur bei Bauabschnitten)
		if (isBauabschnitt()) {
			Float vmin = null;
			Float vreise = null;
			final NumberValue vminvalue = getData()
					.getItem(getDavAttributPrefix() + "GeschwindigkeitsBegrenzungMinimal").asUnscaledValue();
			if (!vminvalue.isState()) {
				vmin = vminvalue.floatValue();
			}
			final NumberValue vreisevalue = getData()
					.getItem(getDavAttributPrefix() + "GeschwindigkeitsBegrenzungFuerReisezeit").asUnscaledValue();
			if (!vreisevalue.isState()) {
				vreise = vreisevalue.floatValue();
			}

			if (vmin != null || vreise != null) {
				final OperatorActionExtensionType operatorActionExtensionType = getObjectFactory()
						.createOperatorActionExtensionType();
				final OperatorActionExtended operatorActionExtended = getObjectFactory().createOperatorActionExtended();

				if (vmin != null) {
					operatorActionExtended.setMinimumSpeedLimit(vmin);
				}
				if (vreise != null) {
					operatorActionExtended.setMainSpeedLimit(vreise);
				}

				operatorActionExtensionType.setOperatorActionExtended(operatorActionExtended);
				operatorAction.setOperatorActionExtension(operatorActionExtensionType);
			}
		}

		return situationRecord;
	}
}
