/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.AffectedCarriagewayAndLanes;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.CarriagewayEnum;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.ConfidentialityValueEnum;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.CountryEnum;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.D2LogicalModel;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.Exchange;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.HeaderInformation;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.InformationStatusEnum;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.InternationalIdentifier;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.LaneEnum;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PayloadPublication;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.Point;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PointByCoordinates;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PointCoordinates;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PredefinedLocation;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PredefinedLocationsPublication;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.PredefinedNonOrderedLocationGroup;
import eu.datex2.schema.statisch.predefined.locations.fahrstreifen.SupplementaryPositionalDescription;

/**
 * Konvertierung von statischen Daten (Messquerschnittsverortung) in das
 * DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class FSPredefinedLocation extends StandardKonvertierungStatisch {
	/** Akzeptierte Datenverteiler-Typen. */
	private static final String TYP_MESS_QUERSCHNITT = "typ.messQuerschnitt";

	/**
	 * Konstruktor.
	 * 
	 * @param plugin
	 *            Referenz auf Plugin-Definitions-Klasse.
	 */

	public FSPredefinedLocation(final PluginExport plugin) {
		this.plugin = plugin;
	}

	/***************************************************************************
	 * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
	 * 
	 * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
	 * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf andere
	 * Klassen; eine nominale Gleichheit existiert also nicht.
	 ***************************************************************************
	 */

	/**
	 * Gebe den in dieser Klasse verwendeten JAXBContext zurück.
	 * 
	 * @throws JAXBException
	 *             JAXBException
	 * @see javax.xml.bind.JAXBContext#newInstance(String)
	 *
	 * @return JAXBContext Verwendeter Kontext
	 */

	@Override
	public JAXBContext getJAXBContext() throws JAXBException {
		if (jaxbContext == null) {
			jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
		}
		return jaxbContext;
	}

	/**
	 * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language,
	 * PublicationCreator).
	 * 
	 * @param d2l
	 *            D2LogicalModel
	 * @param payload
	 *            Verwendeter Payload
	 */

	private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
		InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
		internationalIdentifier.setCountry(CountryEnum.DE);
		internationalIdentifier.setNationalIdentifier(nationalIdentifier);

		// Setze Exchange

		Exchange exchange = new Exchange();
		exchange.setSupplierIdentification(internationalIdentifier);
		d2l.setExchange(exchange);

		// Setze Language
		payload.setLang("DE");

		payload.setPublicationCreator(internationalIdentifier);
	}

	/**
	 * Setzt Koordinaten aus Data in ein PointCoordinates Objekt.
	 * 
	 * @param data
	 *            Data
	 * @return PointCoordinates
	 */

	private PointCoordinates setzeKoordinaten(final Data data) {
		PointCoordinates pointCoordinates = new PointCoordinates();
		pointCoordinates.setLatitude(WerkzeugSammlung.getLatitude(data));
		pointCoordinates.setLongitude(WerkzeugSammlung.getLongitude(data));
		return pointCoordinates;
	}

	/***********************************************************************
	 * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
	 ***********************************************************************
	 */

	/** {@inheritDoc} */
	@Override
	public Object getDatexII(final Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options) {
		boolean versionFree = WerkzeugSammlung.isVersionFree(options);
		String newVersion = WerkzeugSammlung.erzeugeVersionsString(versionFree);

		D2LogicalModel d2l = new D2LogicalModel();
		PredefinedLocationsPublication payload = new PredefinedLocationsPublication();

		setzeNationalIdentifier(d2l, payload);

		HeaderInformation headerInformation = new HeaderInformation();
		headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
		headerInformation.setInformationStatus(InformationStatusEnum.REAL);
		payload.setHeaderInformation(headerInformation);

		if (versionFree) {
			payload.setPublicationTime(WerkzeugSammlung.konvertiereDatum(0));
		} else {
			Date datum = new Date();
			payload.setPublicationTime(WerkzeugSammlung.konvertiereDatum(datum));
		}

		PredefinedNonOrderedLocationGroup predefinedNonOrderedLocationGroup = new PredefinedNonOrderedLocationGroup();
		predefinedNonOrderedLocationGroup.setId(locationGroupId);
		predefinedNonOrderedLocationGroup.setVersion(newVersion);

		// Map mit allen Fahrstreifen als SystemObject und den PIDs als keys
		Map<String, SystemObject> mapPidAllerFahrStreifen = getFahrStreifen(objekteAtgAspListe);

		List<SystemObject> listeMq = getMessQuerschnitte();
		for (SystemObject messQuerSchnitte : listeMq) {
			try {
				AttributeGroup attributGruppe = messQuerSchnitte.getDataModel()
						.getAttributeGroup("atg.punktKoordinaten");
				Data dataMessQuerSchnitte = messQuerSchnitte.getConfigurationData(attributGruppe);
				if (dataMessQuerSchnitte != null) {
					boolean hasKoordinaten = false;
					String posLat = dataMessQuerSchnitte.getScaledValue("y").getValueText();
					String posLong = dataMessQuerSchnitte.getScaledValue("x").getValueText();
					float fLat = 0;
					float fLong = 0;
					fLat = Float.parseFloat(posLat.replace(",", "."));
					fLong = Float.parseFloat(posLong.replace(",", "."));
					hasKoordinaten = true;
					LOGGER.fine(logPrefix + " " + fLat + " " + fLong);
					if (hasKoordinaten) {
						ObjectSet objectSet = ((ConfigurationObject) messQuerSchnitte).getObjectSet("FahrStreifen");
						List<SystemObject> fslist = objectSet.getElements();
						for (SystemObject fahrStreifenAmMessQuerschnitt : fslist) {
							// Gucken, ob der Fahrstreifen vom Messquerschnitt in der Fahrstreifenliste
							// auftaucht
							if (mapPidAllerFahrStreifen.containsKey(fahrStreifenAmMessQuerschnitt.getPid())) {

								SystemObject sysObj = mapPidAllerFahrStreifen
										.get(fahrStreifenAmMessQuerschnitt.getPid());
								AttributeGroup aGruppe = sysObj.getDataModel().getAttributeGroup("atg.fahrStreifen");
								Data dataFahrStreifen = sysObj.getConfigurationData(aGruppe);
								PredefinedLocation predefinedLocation = new PredefinedLocation();
								predefinedLocation.setVersion(newVersion);
								predefinedLocation.setId(fahrStreifenAmMessQuerschnitt.getPid());
								Point point = new Point(); // extends NetworkLocation -> extends Location
								SupplementaryPositionalDescription supplementaryPositionalDescription = new SupplementaryPositionalDescription();
								AffectedCarriagewayAndLanes affectedCarriagewayAndLanes = setFahrstreifen(
										dataFahrStreifen);
								supplementaryPositionalDescription.getAffectedCarriagewayAndLanes()
										.add(affectedCarriagewayAndLanes);
								point.setSupplementaryPositionalDescription(supplementaryPositionalDescription);
								PointByCoordinates pointByCoordinates = new PointByCoordinates();
								PointCoordinates pointCoorninates = new PointCoordinates();

								pointCoorninates.setLatitude(fLat);
								pointCoorninates.setLongitude(fLong);
								pointByCoordinates.setPointCoordinates(pointCoorninates);
								point.setPointByCoordinates(pointByCoordinates);
								predefinedLocation.setLocation(point);
								predefinedNonOrderedLocationGroup.getPredefinedLocations().add(predefinedLocation);

							} // pidFsMap.containsKey(mqFs.getPid())
						} // for(ystemObject mqFs : fslist)
					} // hasKoordinaten
				} // d != null
			} catch (Exception e) {
				LOGGER.fine(logPrefix + "Überspringe Eintrag: ", e);
			}
		} // for (SystemObject messQuerSchnitte : los)
		if (predefinedNonOrderedLocationGroup.getPredefinedLocations().size() < 2) {
			return null;
		} else {
			if (!versionFree) {
				this.version = newVersion;
				try {
					plugin.getPersistenzModul().putData(KEY_PREDEFINED_LOCATION_VERSION, version);
				} catch (Exception e) {
					LOGGER.warning(logPrefix + "PredefinedLocationVersion kann nicht persistiert werden.", e);
				}
			}
			payload.getPredefinedLocationContainers().add(predefinedNonOrderedLocationGroup);
			d2l.setPayloadPublication(payload);
			return d2l;
		}
	}

	/**
	 * Umwandlung FahrStreifen-Angabe =&gt; DATEX-II.
	 * 
	 * @param data
	 *            Datenverteiler-Daten mit Feld "Lage"
	 * @return DATEX-II Struktur
	 */

	protected AffectedCarriagewayAndLanes setFahrstreifen(final Data data) {
		AffectedCarriagewayAndLanes affectedCarriagewayAndLanes = new AffectedCarriagewayAndLanes();
		affectedCarriagewayAndLanes.setCarriageway(CarriagewayEnum.MAIN_CARRIAGEWAY);
		String fahrstreifen = data.getTextValue("Lage").getValueText();
		switch (fahrstreifen.trim()) {
		case "HFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_1);
			break;
		case "1ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_2);
			break;
		case "2ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_3);
			break;
		case "3ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_4);
			break;
		case "4ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_5);
			break;
		case "5ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_6);
			break;
		case "6ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_7);
			break;
		case "7ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_8);
			break;
		case "8ÜFS":
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.LANE_9);
			break;
		default:
			affectedCarriagewayAndLanes.getLanes().add(LaneEnum.RIGHT_LANE);
			break;
		}
		return affectedCarriagewayAndLanes;
	}

	/**
	 * Hole alle Messquerschnitte zur Zuordnung zu FahrStreifen.
	 * 
	 * @return Liste mit SystemObjekten
	 */

	protected List<SystemObject> getMessQuerschnitte() {
		ClientDavInterface davConnection = plugin.getDaVConnection();
		ObjekteAtgAsp objekteAtgAsp = new ObjekteAtgAsp(davConnection, TYP_MESS_QUERSCHNITT);
		List<SystemObject> sol = objekteAtgAsp.getSystemObjekte();
		return sol;
	}

	@Override
	public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
		initialisiereBasisFelder();

		// Modifiziere Liste der Systemobjekte: Entferne alle Objekte, bei denen keine
		// Koordinatenangabe möglich ist.

		Map<String, SystemObject> mapPidAllerFahrStreifen = getFahrStreifen(objects);
		List<SystemObject> listeNeueSystemObjekte = new ArrayList<>();
		List<SystemObject> listeMq = getMessQuerschnitte();
		for (SystemObject messQuerSchnitte : listeMq) {
			try {
				AttributeGroup attributGruppe = messQuerSchnitte.getDataModel()
						.getAttributeGroup("atg.punktKoordinaten");
				Data dataMessQuerSchnitte = messQuerSchnitte.getConfigurationData(attributGruppe);

				try {
					setzeKoordinaten(dataMessQuerSchnitte); // throws exception if
															// anything
															// goes wrong
				} catch (NullPointerException e) {
					LOGGER.fine(logPrefix + "Koordinaten konnten nicht ermittelt werden. Überspringe Eintrag " + messQuerSchnitte.getPid());
					throw new RuntimeException(e);
				}

				ObjectSet objectSet = ((ConfigurationObject) messQuerSchnitte).getObjectSet("FahrStreifen");
				List<SystemObject> fslist = objectSet.getElements();
				for (SystemObject fahrStreifenAmMessQuerschnitt : fslist) {
					// Gucken, ob der Fahrstreifen vom Messquerschnitt in der Fahrstreifenliste
					// auftaucht
					if (mapPidAllerFahrStreifen.containsKey(fahrStreifenAmMessQuerschnitt.getPid())) {
						SystemObject sysObj = mapPidAllerFahrStreifen.get(fahrStreifenAmMessQuerschnitt.getPid());
						listeNeueSystemObjekte.add(sysObj);
					} // pidFsMap.containsKey(mqFs.getPid())
				} // for(ystemObject mqFs : fslist)
			} catch (Exception e) {
				LOGGER.fine(logPrefix + "Überspringe Eintrag " + messQuerSchnitte.getPid(), e);
			}
		}

		for (ObjekteAtgAsp objekteAtgAsp : objects) {
			objekteAtgAsp.getSystemObjekte().clear();
			objekteAtgAsp.getSystemObjekte().addAll(listeNeueSystemObjekte);
		}
	}

	/**
	 * Erzeuge Mapping Pid von Fahrstreifen und Fahrstreifen-Objekte.
	 * 
	 * @param objects
	 *            Objekte
	 * @return Mapping
	 */

	private static Map<String, SystemObject> getFahrStreifen(final Collection<ObjekteAtgAsp> objects) {
		Map<String, SystemObject> mapPidAllerFahrStreifen = new HashMap<>();
		for (ObjekteAtgAsp objekteAtgAsp : objects) {
			for (SystemObject fahrStreifen : objekteAtgAsp.getSystemObjekte()) {
				mapPidAllerFahrStreifen.put(fahrStreifen.getPid(), fahrStreifen);
			}
		}
		return mapPidAllerFahrStreifen;
	}

}
