/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCDirection;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCDirectionEnum;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCLocation;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCMethod2Linear;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCMethod2PrimaryPointLocation;
import eu.datex2.schema.statisch.predefined.locations.segmente.AlertCMethod2SecondaryPointLocation;
import eu.datex2.schema.statisch.predefined.locations.segmente.ConfidentialityValueEnum;
import eu.datex2.schema.statisch.predefined.locations.segmente.CountryEnum;
import eu.datex2.schema.statisch.predefined.locations.segmente.D2LogicalModel;
import eu.datex2.schema.statisch.predefined.locations.segmente.Exchange;
import eu.datex2.schema.statisch.predefined.locations.segmente.HeaderInformation;
import eu.datex2.schema.statisch.predefined.locations.segmente.InformationStatusEnum;
import eu.datex2.schema.statisch.predefined.locations.segmente.InternationalIdentifier;
import eu.datex2.schema.statisch.predefined.locations.segmente.Linear;
import eu.datex2.schema.statisch.predefined.locations.segmente.PayloadPublication;
import eu.datex2.schema.statisch.predefined.locations.segmente.PredefinedLocation;
import eu.datex2.schema.statisch.predefined.locations.segmente.PredefinedLocationsPublication;
import eu.datex2.schema.statisch.predefined.locations.segmente.PredefinedNonOrderedLocationGroup;

/**
 * Konvertierung von statischen Daten (Verortung der Segmente) in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class SDPredefinedLocation extends StandardKonvertierungStatisch {
   /** Schlüssel für AlertC Table Nummer. */
   private static final String KEY_ALERTC_TABLE_NR = "AlertC-Table-Number";
   
   /** Schlüssel für AlertC Table Version. */
   private static final String KEY_ALERTC_TABLE_VERSION = "AlertC-Table-Version";
   
   /** Schlüssel für AlertC Table Country Code. */
   private static final String KEY_ALERTC_TABLE_COUNTRY = "AlertC-Table-Country";

   /** Fehlerwert. */
   private static final int SMALL_MINUS_INT = -1;
   
//   private static final String ASPEKT = "störfallVerfahrenOLSIM1";

   /** AlertC Table Nummer. */
   private String alertCTableNumber = "111130";
   
   /** AlertC Table Version. */
   private String alertCTableVersion = "11.0";
   
   /** AlertC Table Country Code. */
   private String alertCTableCountry = "DE";
   
   /**
    * Konstruktor.
    * 
    * @param plugin Referenz auf Plugin-Definitions-Klasse.
    */
  
  public SDPredefinedLocation(final PluginExport plugin) {
    this.plugin = plugin;
  }


  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurück. 
   * 
   * @throws JAXBException JAXBException
   * @see javax.xml.bind.JAXBContext#newInstance(String)
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Verwendeter Payload
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    
    // Setze Exchange
    
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }
  
  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options) {
    boolean versionFree = WerkzeugSammlung.isVersionFree(options);
    String newVersion = WerkzeugSammlung.erzeugeVersionsString(versionFree);

    D2LogicalModel d2l = new D2LogicalModel();
    PredefinedLocationsPublication payload = new PredefinedLocationsPublication();
    
    setzeNationalIdentifier(d2l, payload);

    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setHeaderInformation(headerInformation);

    payload.setPublicationTime(WerkzeugSammlung.erzeugeVersionsdatum(versionFree));

    PredefinedNonOrderedLocationGroup predefinedNonOrderedLocationGroup = new PredefinedNonOrderedLocationGroup();
    predefinedNonOrderedLocationGroup.setVersion(newVersion);
    predefinedNonOrderedLocationGroup.setId(locationGroupId);

    Set<SystemObject> uniqueList = new HashSet<>();

    for (ObjekteAtgAsp objekteAtgAsp : objekteAtgAspListe) {
      uniqueList.addAll(objekteAtgAsp.getSystemObjekte());
    }

    for (SystemObject mq : uniqueList) {
      try {
        AttributeGroup attributGruppe = mq.getDataModel().getAttributeGroup("atg.straßenSegmentLclInfo");
        Data d = mq.getConfigurationData(attributGruppe);

        if (d != null) {  
          String tmcPunktAnfang = WerkzeugSammlung.getStringValue(d, "TmcPunktAnfang");
          String tmcPunktEnde = WerkzeugSammlung.getStringValue(d, "TmcPunktEnde");
          int tmcPunktAnfangLongValue = getTmcIntValue(tmcPunktAnfang, SMALL_MINUS_INT);
          int tmcPunktEndeLongValue = getTmcIntValue(tmcPunktEnde, SMALL_MINUS_INT);

          if (tmcPunktAnfangLongValue > SMALL_MINUS_INT && tmcPunktEndeLongValue > SMALL_MINUS_INT) {
            AttributeGroup attrGruppe = mq.getDataModel().getAttributeGroup("atg.äußeresStraßenSegment");
            Data dat = mq.getConfigurationData(attrGruppe);
            String richtung = WerkzeugSammlung.getStringValue(dat, "TmcRichtung");
            AlertCDirection alertCDirection = new AlertCDirection();
            if (richtung.trim().equalsIgnoreCase("positiv")) {
              alertCDirection.setAlertCDirectionCoded(AlertCDirectionEnum.POSITIVE);
            } else if (richtung.trim().equalsIgnoreCase("negativ")) {
              alertCDirection.setAlertCDirectionCoded(AlertCDirectionEnum.NEGATIVE);
            } else {
              alertCDirection.setAlertCDirectionCoded(AlertCDirectionEnum.UNKNOWN);
            }
            PredefinedLocation predefinedLocation = new PredefinedLocation();
            predefinedLocation.setVersion(newVersion);
            String pid = mq.getPid();
            predefinedLocation.setId(pid);
            predefinedLocation.setLocation(null);
            Linear linear = new Linear();
            AlertCMethod2Linear alertCMethod2Linear = new AlertCMethod2Linear();

            alertCMethod2Linear.setAlertCLocationCountryCode(alertCTableCountry);
            alertCMethod2Linear.setAlertCLocationTableNumber(alertCTableNumber);
            alertCMethod2Linear.setAlertCLocationTableVersion(alertCTableVersion);
            AlertCMethod2PrimaryPointLocation alertCMethod2PrimaryPointLocation = 
                new AlertCMethod2PrimaryPointLocation();
            AlertCMethod2SecondaryPointLocation alertCMethod2SecondaryPointLocation = 
                new AlertCMethod2SecondaryPointLocation();
            AlertCLocation alertCLocationAnfang = new AlertCLocation();
            AlertCLocation alertCLocationEnde = new AlertCLocation();
            alertCLocationAnfang.setSpecificLocation(BigInteger.valueOf(tmcPunktAnfangLongValue));
            alertCLocationEnde.setSpecificLocation(BigInteger.valueOf(tmcPunktEndeLongValue));
            alertCMethod2PrimaryPointLocation.setAlertCLocation(alertCLocationEnde);
            alertCMethod2SecondaryPointLocation.setAlertCLocation(alertCLocationAnfang);
            alertCMethod2Linear.setAlertCMethod2PrimaryPointLocation(alertCMethod2PrimaryPointLocation);
            alertCMethod2Linear.setAlertCMethod2SecondaryPointLocation(alertCMethod2SecondaryPointLocation);

            alertCMethod2Linear.setAlertCDirection(alertCDirection);
            linear.setAlertCLinear(alertCMethod2Linear);
            predefinedLocation.setLocation(linear);
            predefinedNonOrderedLocationGroup.getPredefinedLocations().add(predefinedLocation);
          }
        }
      } catch (Exception e) {
        LOGGER.error(logPrefix + "Verwerfe einen Datensatz.", e);
      }
    }
    if (predefinedNonOrderedLocationGroup.getPredefinedLocations().size() < 2) {
      return null;
    } else {
      if (!versionFree) {
        this.version = newVersion;
        try {
          plugin.getPersistenzModul().putData(KEY_PREDEFINED_LOCATION_VERSION, version);
        } catch (Exception e) {
          LOGGER.warning(logPrefix + "PredefinedLocationVersion kann nicht persistiert werden.", e);
        }
      }
      payload.getPredefinedLocationContainers().add(predefinedNonOrderedLocationGroup);
      d2l.setPayloadPublication(payload);
      return d2l;
    }
  }
  
  /**
   * Ermittle den TMC-Code.
   * 
   * @param tmcString Pid des LocationCode-Objekts
   * @param defaultValue Standard-Wert, fall LocationCode nicht gefunden wird.
   * @return Location-Code als int
   */

  private static int getTmcIntValue(final String tmcString, final int defaultValue) {
    int res = defaultValue;	
    String svalue = tmcString.replace("tmclc.", "").trim();
    try {
      res = Integer.parseInt(svalue);
    } catch (NumberFormatException e) {
      res = defaultValue;
    }


    return res;
  }

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    initialisiereBasisFelder();
    
    alertCTableNumber = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_NR);
    alertCTableVersion = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_VERSION);
    alertCTableCountry = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_ALERTC_TABLE_COUNTRY);    
  }  
}